# © 2020 - today Numigi (tm) and all its contributors (https://bit.ly/numigiens)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl).
from typing import List

import re

task_details_from_tag_regex = re.compile(
    # r"(^|\s+)(?P<task_tag>[Tt][Aa]#?(?P<id>\d+))(\s+|$)"
    r"(^|\s+)(?P<task_tag>[Tt][Aa]#?(?P<id>\d+)[^\w\s]*)(\s+|$)"
)


def find_ids(text: str, ids=None) -> List[str]:
    ids = ids or set([])
    id_, task_tag = _find_task_details(text)

    if id_:
        ids.add(id_)
        cleaned_text = _remove_task_from_text(text, task_tag)
        ids = find_ids(cleaned_text, ids)

    return sorted(list(ids))


def _find_task_details(text: str) -> (str, str):
    no_parentheses = re.sub(r"([()])", " ", text)
    res = re.search(task_details_from_tag_regex, no_parentheses)

    if res:
        group_dict = res.groupdict()
        return group_dict["id"], group_dict["task_tag"]

    return "", ""


def _remove_task_from_text(text: str, task_tag: str) -> str:
    return re.sub(task_tag, "", text)



# © 2020 - today Numigi (tm) and all its contributors (https://bit.ly/numigiens)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl).

import pytest

from odoo_regex_set import task


@pytest.mark.parametrize(
    "input_,expected",
    [
        ("TA#6789", ["6789"]),
        ("TA#1234 TA#4567", ["1234", "4567"]),
        ("TA#1234, TA#4567", ["1234", "4567"]),
        ("TA#1234. TA#4567", ["1234", "4567"]),
        ("TA#1234! TA#4567!", ["1234", "4567"]),
        ("TA#1234 TA#4567 ta0987", ["0987", "1234", "4567"]),
        ("(TA#6890 TA#4567)", ["4567", "6890"]),
        (
                """
        TA#1234
        TA#4567
        ta0987
        """,
                ["0987", "1234", "4567"]
        )  # multilines,
    ]
)
def test_find_ids_found(input_, expected):
    """ Test the function accepts any numbers of tags"""
    assert task.find_ids(input_) == expected


@pytest.mark.parametrize(
    "input_",
    [
        "",  # empty string
    ]
)
def test_find_ids_empty(input_):
    assert not task.find_ids(input_)


@pytest.mark.parametrize(
    "input_,expected",
    [
        ("TA#6789", ("6789", "TA#6789")),
        ("TA#6789 TA#6789", ("6789", "TA#6789")),
        ("ta#6752", ("6752", "ta#6752")),
        ("TA6785", ("6785", "TA6785")),
        ("ta6751", ("6751", "ta6751")),
        ("Some text TA#6787 followed by some text", ("6787", "TA#6787")),
        ("(TA#6752)", ("6752", "TA#6752")),
        ("Une phrase avec le tag suivi par un point TA#6754", ("6754", "TA#6754"))
    ]
)
def test_find_task_details_success(input_, expected):
    assert task._find_task_details(input_) == expected


@pytest.mark.parametrize(
    "input_",
    [
        "",  # empty string
        "⁣TA#6789⁣",  # invisble spaces
        "6TA#5678",  # a number before the tag
        "tt#5678",  # not the correct tag
        "sometextnospaceTA#5678nospace",  # the tag with some text around without any spaces
        "TA#4567nospace",  # the tag followed by some text without space
        "34567TA4567nospace",  # some numbers a tag and some text after
        "3456TA#3456",  # some numbers then a tag
        "56789-TA#4567-45678",  # tag surrounded by -
        "4567-TA#4567$sometext"  # tag surrounded by special characters
        "^⁣TA#5647"  # tag with a special character before
    ]
)
def test_find_task_details_failure(input_):
    assert task._find_task_details(input_) == ("", "")


@pytest.mark.parametrize(
    "text,task_tag,expected",
    [
        ("", "", ""),
        ("TA#6789", "TA#6789", ""),
        ("TA#6789", "TA#1234", "TA#6789"),
        ("TA#1234 TA#6789", "TA#1234", " TA#6789"),
        ("TA#1234 TA#6789", "TA#6789", "TA#1234 "),
        ("TA#1234 TA#1234", "TA#1234", " "),  # remove the task tags but not the space :)
    ]
)
def test_remove_task_from_text(text, task_tag, expected):
    assert task._remove_task_from_text(text, task_tag) == expected



#!/usr/bin/env python

from setuptools import find_packages, setup
from odoo_regex_set import manifest

setup(
    name=manifest.name,
    use_scm_version=True,
    setup_requires=['setuptools_scm', 'pytest_runner'],
    description=manifest.description,
    author=manifest.author,
    author_email=manifest.email,
    url=manifest.url,
    packages=find_packages(exclude=('tests',)),
    install_requires=[
    ],
    tests_require=[
        'pytest<5.0.0',
        'pytest-cov',
        'pytest-random-order',
    ],
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)
